Class: LLVM::Function
- Inherits:
-
GlobalValue
- Object
- Value
- User
- Constant
- GlobalValue
- LLVM::Function
- Defined in:
- lib/llvm/analysis.rb,
lib/llvm/core/value.rb
Defined Under Namespace
Classes: AttributeCollection, BasicBlockCollection, ParameterCollection
Instance Method Summary collapse
-
#add_attribute(attr, index = -1)) ⇒ Object
Adds attr to this value’s attributes.
- #attribute_count ⇒ Object
- #attributes ⇒ Object
-
#basic_blocks ⇒ Object
Returns an Enumerable of the BasicBlocks in this function.
-
#call_conv=(conv) ⇒ Object
Sets the function’s calling convention and returns it.
- #function_attributes ⇒ Object
- #function_type ⇒ Object
- #gc ⇒ Object
- #gc=(name) ⇒ Object
- #inspect ⇒ Object
- #param_attributes(index) ⇒ Object
-
#params ⇒ Object
Returns an Enumerable of the parameters in the function.
-
#remove_attribute(attr, index = -1)) ⇒ Object
Removes the given attribute from the function.
- #return_attributes ⇒ Object
- #return_type ⇒ Object
-
#type ⇒ Object
In LLVM 15, not overriding this yields a pointer type instead of a function type.
- #valid? ⇒ Boolean
-
#verify ⇒ true, false
Verify that a function is valid.
-
#verify! ⇒ true, false
Verify that a function is valid, and abort the process if not.
Methods inherited from GlobalValue
#alignment, #alignment=, #declaration?, #dll_storage_class, #dll_storage_class=, #global_constant=, #global_constant?, #initializer, #initializer=, #linkage, #linkage=, #section, #section=, #unnamed_addr=, #unnamed_addr?, #visibility, #visibility=
Methods inherited from Constant
#bitcast_to, #gep, null, null_ptr, #ptr_to_int, undef
Methods inherited from User
Methods inherited from Value
#allocated_type, #constant?, #dump, from_ptr, from_ptr_kind, #kind, #name, #name=, #null?, to_ptr, #to_s, type, #undefined?
Methods included from PointerIdentity
Instance Method Details
#add_attribute(attr, index = -1)) ⇒ Object
Adds attr to this value’s attributes.
822 823 824 |
# File 'lib/llvm/core/value.rb', line 822 def add_attribute(attr, index = -1) AttributeCollection.new(self, index).add(attr) end |
#attribute_count ⇒ Object
831 832 833 |
# File 'lib/llvm/core/value.rb', line 831 def attribute_count function_attributes.count end |
#attributes ⇒ Object
835 836 837 |
# File 'lib/llvm/core/value.rb', line 835 def attributes function_attributes.to_a end |
#basic_blocks ⇒ Object
Returns an Enumerable of the BasicBlocks in this function.
804 805 806 |
# File 'lib/llvm/core/value.rb', line 804 def basic_blocks @basic_block_collection ||= BasicBlockCollection.new(self) end |
#call_conv=(conv) ⇒ Object
Sets the function’s calling convention and returns it.
798 799 800 801 |
# File 'lib/llvm/core/value.rb', line 798 def call_conv=(conv) C.set_function_call_conv(self, conv) conv end |
#function_attributes ⇒ Object
839 840 841 |
# File 'lib/llvm/core/value.rb', line 839 def function_attributes AttributeCollection.new(self, -1) end |
#function_type ⇒ Object
808 809 810 |
# File 'lib/llvm/core/value.rb', line 808 def function_type Type.from_ptr(C.get_element_type(self), :function) end |
#gc ⇒ Object
1000 1001 1002 |
# File 'lib/llvm/core/value.rb', line 1000 def gc C.get_gc(self) end |
#gc=(name) ⇒ Object
996 997 998 |
# File 'lib/llvm/core/value.rb', line 996 def gc=(name) C.set_gc(self, name) end |
#inspect ⇒ Object
1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 |
# File 'lib/llvm/core/value.rb', line 1004 def inspect { signature: to_s.lines[attribute_count == 0 ? 0 : 1], type: type.to_s, attributes: attribute_count, valid: valid?, blocks: basic_blocks.size, lines: to_s.lines.size, }.to_s end |
#param_attributes(index) ⇒ Object
847 848 849 |
# File 'lib/llvm/core/value.rb', line 847 def param_attributes(index) AttributeCollection.new(self, index) end |
#params ⇒ Object
Returns an Enumerable of the parameters in the function.
963 964 965 |
# File 'lib/llvm/core/value.rb', line 963 def params @parameter_collection ||= ParameterCollection.new(self) end |
#remove_attribute(attr, index = -1)) ⇒ Object
Removes the given attribute from the function.
827 828 829 |
# File 'lib/llvm/core/value.rb', line 827 def remove_attribute(attr, index = -1) AttributeCollection.new(self, index).remove(attr) end |
#return_attributes ⇒ Object
843 844 845 |
# File 'lib/llvm/core/value.rb', line 843 def return_attributes AttributeCollection.new(self, 0) end |
#return_type ⇒ Object
817 818 819 |
# File 'lib/llvm/core/value.rb', line 817 def return_type type.return_type end |
#type ⇒ Object
In LLVM 15, not overriding this yields a pointer type instead of a function type
813 814 815 |
# File 'lib/llvm/core/value.rb', line 813 def type function_type end |
#valid? ⇒ Boolean
49 50 51 |
# File 'lib/llvm/analysis.rb', line 49 def valid? verify end |
#verify ⇒ true, false
Verify that a function is valid.
39 40 41 |
# File 'lib/llvm/analysis.rb', line 39 def verify do_verification(:return_status) end |
#verify! ⇒ true, false
Verify that a function is valid, and abort the process if not.
45 46 47 |
# File 'lib/llvm/analysis.rb', line 45 def verify! do_verification(:abort_process) end |